const config = require("config");

class Zammad {

    /**
     * Creates a notification in the configured Zammad instance
     * 
     * @param {number} token_amount
     * @param {number} $price
     * @param {string} $payment_method
     * 
     * @returns {Promise<boolean>}
     */
    static async CREATE_NOTIFICATION(token_amount, price, payment_method) {
        // Check if notifications are turned on
        try {
            if (config.get("app.zammad.enabled")) {
                let zammad_url = config.get("app.zammad.url") + "/api/v1/ticket_articles";

                let zammad_data = {
                    "ticket_id": config.get("app.zammad.notification_ticket_id"),
                    "subject": "Neue Bestellung",
                    "body": `Tokens: ${token_amount}\nBetrag: ${price.toFixed(2)}€\nZahlungsart: ${payment_method}`,
                    "content_type": "text/plain",
                    "type": "web",
                    "internal": false,
                    "sender": "Agent",
                    "time_unit": "15"
                }
                return fetch(zammad_url, {
                    method: "POST",
                    headers: {
                        "Content-Type": "application/json",
                        "Authorization": `Token ${config.get("app.zammad.api_key")}`
                    },
                    body: JSON.stringify(zammad_data)
                }).then(response => {
                    return true;
                }).catch(e => {
                    console.error(e);
                    return false;
                })
            } else {
                return true;
            }
        } catch (e) {
            console.error(e);
            return false;
        }
    }
}

module.exports = Zammad;