var express = require("express");
var router = express.Router({ mergeParams: true });

const config = require("config");
const { body, validationResult } = require("express-validator");
const Receipt = require("../../app/Receipt");

router.get("/", (req, res) => {
  req.data.order.invoice = {
    params: {
      name: req.query.name || "",
      email: req.query.email || "",
      address: req.query.address || "",
    },
    create_invoice_url: `${res.locals.baseDir
      }/key/${req.data.key.key.get_key()}/orders/${req.data.order.payment.public_id
      }/invoice/create`,
    errors: {},
  };

  res.render("key", req.data);
});

router.post(
  "/*",
  body("email").isEmail({ domain_specific_validation: true }),
  body("address").isLength({ max: 1000 }),
  body("name").isLength({ max: 500 }),
  body("company").isLength({ max: 500 }),
  (req, res, next) => {
    req.data.order.invoice = {
      params: {
        company: req.body.company || "",
        name: req.body.name || "",
        email: req.body.email || "",
        address: req.body["address"] || "",
      },
      errors: {},
    };

    const errors = validationResult(req);
    if (!errors.isEmpty()) {
      errors.array().forEach((error) => {
        req.data.order.invoice.errors[error.param] = error.msg;
      });
      res.render("key", req.data);
      return;
    }
    next();
  }
);

router.post("/", (req, res) => {
  let moderation_params = {
    order: req.data.order.payment.public_id,
    payment_reference: req.data.order.payment_reference.public_id,
    name: req.data.order.invoice.params.name,
    email: req.data.order.invoice.params.email,
    address: req.data.order.invoice.params.address,
  };
  req.data.order.invoice.params = moderation_params;
  req.data.order.invoice.moderation_url = `${res.locals.baseDir
    }/admin/payments/receipt?${new URLSearchParams(
      moderation_params
    ).toString()}`;

  // Render the message
  let ejs = require("ejs"),
    fs = require("fs"),
    template = fs.readFileSync(
      `${__dirname}/../../views/orders/invoice_message.ejs`,
      "utf-8"
    );

  let message = ejs.render(template, req.data);

  // No validation errors. Try to create a new Ticket
  return fetch(`${config.get("app.osticket.url")}/api/tickets.json`, {
    method: "post",
    headers: {
      "X-API-Key": config.get("app.osticket.api_key"),
      "Content-Type": "application/json",
    },
    body: JSON.stringify({
      alert: true,
      autorespond: false,
      source: "API",
      name: req.data.order.invoice.params.name,
      email: req.data.order.invoice.params.email,
      subject: `MetaGer Schlüssel: Rechnung (${req.data.order.payment.public_id})`,
      message: `data:text/html;charset=utf-8,${message}`,
      topicId: 12, // ToDo change topic for english autoresponder
    }),
  })
    .then((response) => {
      if (response.status != 201) {
        console.error(response.body);
        throw "Fehler beim Erstellen der Benachrichtigung,";
      } else {
        req.data.order.invoice.success = true;
        res.render("key", req.data);
      }
    })
    .catch((reason) => {
      console.log(reason);
      req.data.order.invoice.errors["send_email"] =
        "Fehler beim Erstellen der Benachrichtigung,";
      res.render("key", req.data);
    });
});

router.get("/download", (req, res) => {
  if (req.data.order.payment.receipt_id === null) {
    res.locals.error = { status: 404 };
    res.locals.message = "Receipt not found";
    res.status(404).render("error");
  }
  return Receipt.LOAD_RECEIPT_FROM_INTERNAL_ID(req.data.order.payment.receipt_id).then(receipt => {
    res.type("pdf").header({
      "Content-Disposition": `inline; filename=${receipt.public_id}.pdf`
    }).send(Buffer.from(receipt.receipt.toString(), "base64"));
  }).catch(reason => {
    console.error(reason);
    res.locals.error = { status: 404 };
    res.locals.message = "Receipt not found";
    res.status(404).render("error");
  });
});

module.exports = router;
